/*
Presto - An Allegro tilemap library
Copyright (C) 2008  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


functions.h: functions to set up or change a tilemap
*/


//changes the animation that the tile follows.  -1 tostop animation
//also resets the tile index to 0 for the start of the new animation
void tile_change_anim(TILEMAP *tilemap, unsigned int x, unsigned int y, int id);

//changes the animation step that the tile is on if animated, or changes the current sprite if static.
//In the latter case, index refers to the id of the bitmap in the pool
//NOTE: If the tile is animated and synced to the frameclock, this function will have no effect
void tile_change_index(TILEMAP * tilemap, unsigned int x, unsigned int y, unsigned int index);

//changes the loop parameter for a tile.  Set loop for -1 for infinite loop, 0 to stop looping, or X where X>0 to loop X times.
void tile_set_loop(TILEMAP * tilemap, unsigned int x, unsigned int y, int loop);

//sets whether the tile should sync with the frameclock.  0==No, !0=yes
void tile_set_sync(TILEMAP * tilemap, unsigned int x, unsigned int y, int sync);

//sets the transparency level of the tile in the pool.  255==opaque, 0==transparent
//-2==ignore the alpha channel completely (no transparency)
//-1==read alpha values from the file (but this has no effect after the bitmap is loaded)
void tile_set_trans(TILEMAP * tilemap, unsigned int index, int trans);

//changes an animation sequence currently stored in the tilemap or adds a new one if index <0
//Returns the index of the animation in the animation list or <0 on failure
int tilemap_change_anim(TILEMAP *tilemap, int index, unsigned int *order, unsigned int *delays, int loop, unsigned int count);

//changes a bitmap at index in the pool, or adds a new one if index <0
//Returns the index of the bitmap or -2 on memory failure, -4 if error loading the bitmap
int tilemap_change_bitmap(TILEMAP *tilemap, int index, char *filename);

//sets up the dimensions of a tilemap
//Returns 0 on success, <0 on failure
int tilemap_create_dims(TILEMAP *tilemap, unsigned int h, unsigned int w);

