/*
Test program for Presto, an Allegro tilemap library
Copyright (C) 2008  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <presto/presto.h>
#include <time.h>
#include <math.h>

#define FPS 30
#define SCROLL_SPEED 4 //scroll the clouds 1 pixel every frame at 30 FPS
#define SHIP_SPEED 5
#define BULLET_SPEED 8
#define CLOCKS_TO_MS(x) ((unsigned long int)(((double)(x))/((double)CLOCKS_PER_SEC)*1000))
#define PI 3.14159


volatile unsigned int sec_elapsed;



void fps_handler(void) {
	
	sec_elapsed=1;
	
}
END_OF_FUNCTION(fps_handler);



typedef struct {
	unsigned long x;
	unsigned long y;
	unsigned int active;
} BULLET;




int main(int argc, char **argv) {
	int color_depth;
	TILEMAP testmap;
	TILEMAP testmap_clouds;
	BITMAP *buf;
	BITMAP *ship;
	BITMAP *bullet;
	//unsigned int testorders[4]={2,3,4,5};
	//unsigned int testdelays[4]={250,250,250,250};
	
	BULLET bullet_field[100];
	
	unsigned long y=1024<<9;
	unsigned long cy=512<<9;
	unsigned long px=((384/2)-24)<<9;
	unsigned long py=(512-64)<<9;
	unsigned int installed=0;
	unsigned long fps_ticks=0;
	long wait_ms;
	clock_t ticks;
	double fps=FPS;
	int i=0;
	int start=0;
	double cloud_period=PI/6;
	int cloud_dir=1;
	TILE *check;
	unsigned int checkx;
	unsigned int checky;
	
	TILEMAP_LIST list;
	TILEMAP *tilemapbuf[1];
	
	tilemapbuf[0]=&testmap;
	
	list.num=1;
	list.list=tilemapbuf;
	
	sec_elapsed=0;
	
	LOCK_VARIABLE(sec_elapsed);
	LOCK_FUNCTION(fps_handler);
	
	
	memset(bullet_field,0,sizeof(bullet_field));
	
	if(allegro_init()) {
		fprintf(stderr,"Error initializing Allegro\n");
		return -1;
	}


	
	
	if(!(color_depth=desktop_color_depth()))
		set_color_depth(32);
	else
		set_color_depth(color_depth);
	
	
		
	if(install_timer()) {
		fprintf(stderr,"Error: Could not init timer\n");
		return -1;
	}

	install_keyboard();
	
	
	if(set_gfx_mode(GFX_AUTODETECT_WINDOWED,384,512,0,0)) {
		fprintf(stderr,"Error: Could not set gfx mode\n");
		return -1;
	}
	
	
	set_window_title("Tilemap Test");
	
	set_color_conversion(COLORCONV_KEEP_ALPHA);


	buf=create_bitmap(SCREEN_W,SCREEN_H);
	ship=load_png("images/ship.png",NULL);
	bullet=load_png("images/bullets.png",NULL);

	if(!buf || !ship) {
		fprintf(stderr,"Error loading image\n");
		return -1;
	}
	
	if(load_tilemap("test.map",&testmap) || load_tilemap("clouds.map",&testmap_clouds)) {
		fprintf(stderr,"Error loading tilemap:\n%s\n",tilemap_get_err());
		return -1;
	}

	
	
	if(install_tilemap(&list)) {
		installed=1;
		fprintf(stderr,"Error installing tilemap:\n%s\n",tilemap_get_err());
	}

	//for ship/bullet transparency
	set_alpha_blender();

	install_int(fps_handler,5000);

	//main loop
	while(1) {
		ticks=clock();
		
		//shameless variable reuse
		wait_ms=(unsigned long)(SCROLL_SPEED*((30*512)/fps));
		
		
		//check for bullet collision with flowers
		for(i=0;i<100;i++) {
			if(bullet_field[i].active) {
				check=testmap.map+((y>>9)>>5)*testmap.width;
				for(checky=0;checky<17;checky++) {
					for(checkx=0;checkx<testmap.width;checkx++,check++) {
						if((((bullet_field[i].y>>9)>>5)==checky) && ((((bullet_field[i].x>>9)>>5)==checkx) || (((bullet_field[i].x>>9)>>5)+1==checkx)) && (!check->orderid)) {
							//we've hit a flower
							tile_change_anim(&testmap,checkx,checky+((y>>9)>>5),1);
							
							bullet_field[i].active=0;
						}
					}
				}
			}
			
		}
		
		
		if(start) {
			//____move stuff____//
			cy-=(wait_ms>cy)?cy:wait_ms;
			y-=(4*wait_ms>y)?y:(4*wait_ms);
			
			
	
			if(y<=0)
				y=1024<<9;
			if(cy<=0)
				cy=512<<9;
				
			
			//move the bullets forward
			for(i=0;i<100;i++) {
				if(bullet_field[i].active) {
					bullet_field[i].y-=((BULLET_SPEED*wait_ms)>bullet_field[i].y)?bullet_field[i].y:(BULLET_SPEED*wait_ms);
					
					if(!bullet_field[i].y)
						bullet_field[i].active=0;
				}
				
			}
		}
		
		
		
		poll_keyboard();
		
		
		
		
		
		if(key[KEY_ESC])
			goto bottom;
		if(key[KEY_SPACE]) //takeoff!
			start=1;
		
		if(start) {
			if(key[KEY_LEFT])
				px-=(wait_ms*SHIP_SPEED>px)?px:wait_ms*SHIP_SPEED;
			else if(key[KEY_RIGHT])
				px+=(((wait_ms*SHIP_SPEED)+px+(ship->w<<9))>=(384<<9))?(384<<9)-px-(ship->w<<9):wait_ms*SHIP_SPEED;
		}
		
			
		if(key[KEY_LCONTROL]) {
			//shoot a bullet
			for(i=0;i<100;i++) {
				if(!bullet_field[i].active) {
					bullet_field[i].active=1;
					bullet_field[i].x=px+((ship->w<<9)/2)-((bullet->w<<9)/2);
					bullet_field[i].y=py-(bullet->h<<9);
					break;
				}
			}
			
			
		}
		
		

		
		fps_ticks++;
		
		if(sec_elapsed) {
			fps=(double)fps_ticks/5;
			printf("FPS: %.2lf\n",fps);
			fps_ticks=0;
			sec_elapsed=0;
		}
		


		update_anim_tiles(&list,0,(y>>9)/32,384/32,(512/32));
		

		//draw the ground
		render_tilemap(buf,&testmap,0,y>>9,12,17);
		
		
		//draw the player ship
		draw_trans_sprite(buf,ship,px>>9,py>>9);
		
		
		//draw any bullets currently on-screen
		for(i=0;i<100;i++) {
			if(bullet_field[i].active) {
				draw_trans_sprite(buf,bullet,bullet_field[i].x>>9,bullet_field[i].y>>9);
			}
		}
		
		if(cloud_period>(5*PI/12))
			cloud_dir=-1;
		if(cloud_period<PI/6)
			cloud_dir=1;
		
		
		cloud_period+=0.02*cloud_dir;
		
		
		
		//fade the clouds in and out
		tile_set_trans(&testmap_clouds,0,sin(cloud_period)*256);
		
		//draw the cloud overlay tilemap
		render_tilemap(buf,&testmap_clouds,0,cy>>9,1,2);
		
		if(!start) {
			textout_centre_ex(buf,font," Shoot the flowers!  ",SCREEN_W/2,SCREEN_H-16,makecol(255,0,0),makecol(0,0,0));
			textout_centre_ex(buf,font,"Press Space to Launch",SCREEN_W/2,SCREEN_H-8,makecol(255,0,0),makecol(0,0,0));
		}
		
		blit(buf,screen,0,0,0,0,SCREEN_W,SCREEN_H);
		

		ticks=CLOCKS_TO_MS(clock()-ticks);
		
		wait_ms=(1000/FPS)-ticks;
		
		if(wait_ms>0)
			rest(wait_ms);

	}
	
	bottom:
	
	free_tilemap(&testmap);
	destroy_bitmap(buf);
	
	if(installed)
		uninstall_tilemap();
	
	return 0;
}

//used by Allegro
END_OF_MAIN();

